@@ -33,16 +33,12 @@
 #include <QMutex>
 #include <QStandardPaths>
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
 // in the desktoptojson binary enable debug messages by default, in the library only warning messages
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
 Q_LOGGING_CATEGORY(DESKTOPPARSER, "kf5.kcoreaddons.desktopparser", QtDebugMsg)
 #else
 Q_LOGGING_CATEGORY(DESKTOPPARSER, "kf5.kcoreaddons.desktopparser", QtWarningMsg)
 #endif
-#else
-Q_LOGGING_CATEGORY(DESKTOPPARSER, "kf5.kcoreaddons.desktopparser")
-#endif
 
 
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
@@ -232,7 +228,7 @@ QByteArray readTypeEntryForCurrentGroup(QFile &df, QByteArray *nextGroup)
         }
         if (line.startsWith(QByteArrayLiteral("Type="))) {
             // TODO: should we also have to accept spaces around equals here?
-            type = line.mid(strlen("Type="));
+            type = line.mid(qstrlen("Type="));
         }
     }
     return type;
@@ -242,10 +238,11 @@ QVector<CustomPropertyDefinition>* parseServiceTypesFile(QString path)
 {
     int lineNr = 0;
     if (QDir::isRelativePath(path)) {
+        const QString originalPath = path;
         path = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
                                QStringLiteral("kservicetypes5/") + path);
         if (path.isEmpty()) {
-            qCCritical(DESKTOPPARSER) << "Could not locate service type file" << path;
+            qCCritical(DESKTOPPARSER) << "Could not locate service type file kservicetypes5/" << originalPath << ", tried" << QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
             return nullptr;
         }
     }
@@ -279,7 +276,7 @@ QVector<CustomPropertyDefinition>* parseServiceTypesFile(QString path)
             qCWarning(DESKTOPPARSER) << "Could not find Type= key in group" << currentGroup;
             continue;
         }
-        QByteArray propertyName = currentGroup.mid(strlen("PropertyDef::"));
+        QByteArray propertyName = currentGroup.mid(qstrlen("PropertyDef::"));
         QVariant::Type type = QVariant::nameToType(typeStr.constData());
         switch (type) {
             case QVariant::String:
@@ -315,10 +312,6 @@ ServiceTypeDefinition::ServiceTypeDefinition(const QVector< CustomPropertyDefini
 {
 }
 
-ServiceTypeDefinition::~ServiceTypeDefinition()
-{
-}
-
 ServiceTypeDefinition ServiceTypeDefinition::fromFiles(const QStringList &paths)
 {
     QVector<CustomPropertyDefinition> defs;
@@ -327,20 +320,28 @@ ServiceTypeDefinition ServiceTypeDefinition::fromFiles(const QStringList &paths)
     // as we might modify the cache we need to acquire a mutex here
     foreach (const QString &serviceType, paths) {
         QMutexLocker lock(&s_serviceTypesMutex);
+
         QVector<CustomPropertyDefinition>* def = s_serviceTypes->object(serviceType);
-        // not found in cache -> we need to parse the file
-        if (!def) {
+
+        if (def) {
+            // in cache but we still must make our own copy
+            defs << *def;
+        }
+        else {
+            // not found in cache -> we need to parse the file
             qCDebug(DESKTOPPARSER) << "About to parse service type file" << serviceType;
             def = parseServiceTypesFile(serviceType);
             if (!def) {
+#ifdef BUILDING_DESKTOPTOJSON_TOOL
+                exit(1); // this is a fatal error when using kcoreaddons_desktop_to_json()
+#else
                 continue;
+#endif
             }
+
+            defs << *def; // This must *precede* insert call, insert might delete
             s_serviceTypes->insert(serviceType, def);
         }
-        // We always have to make a copy of the QVector as it may be deleted from the cache
-        // at any time and could therefore cause dangling pointers
-        defs << *def;
-
     }
     return ServiceTypeDefinition(defs);
 }
@@ -438,7 +439,7 @@ void DesktopFileParser::convertToJson(const QByteArray &key, const ServiceTypeDe
     } else if (key == QByteArrayLiteral("Comment")) {
         kplugin[QStringLiteral("Description")] = value;
     } else if (key.startsWith(QByteArrayLiteral("Comment["))) {
-        kplugin[QStringLiteral("Description") + QString::fromUtf8(key.mid(strlen("Comment")))] = value;
+        kplugin[QStringLiteral("Description") + QString::fromUtf8(key.mid(qstrlen("Comment")))] = value;
     } else if (key == QByteArrayLiteral("Hidden")) {
         DESKTOPTOJSON_VERBOSE_WARNING << "Hidden= key found in desktop file, this makes no sense"
                 " with metadata inside the plugin.";
